function [ineg,muv,V]=spcalc(Gu,p,varargin) 
% SPCALC: compute p.nc.neig(j) EVals of Gu nearp.nc.eigref(j), j=varargin 
%
%  [ineg,muv]=spcalc(Gu,p)
%  [ineg,muv,V]=spcalc(Gu,p) - also compute and return eigenvectors in V
% 
% * muv=  eigenvalues sorted wrt real parts,
% * ineg= number of negative evals.
if (p.sw.spcont~=0)
    [p,spcont,prim]=spreduce(p); 
    Gu=Gu(1:p.nu+p.nc.nq,1:p.nu+p.nc.nq);
end
%figure(4); spy(Gu), pause
vs=size(Gu,1); j=1; if nargin>2; j=varargin{1}; end 
om=p.nc.eigref(j); 
neig=p.nc.neig(j); 
M=getM(p); 
if(p.nc.nq>0) % trivially extend M in case of auxiliary equations
    if isfield(p.fuha,'qMfu'); [qL,qU,qD]=p.fuha.qMfu(p); 
    else [qL,qU,qD]=stanqM(p); 
    end 
    M=[[M(1:p.nu,1:p.nu) qU]; [qL qD]]; 
end

if(strcmp(p.sw.eigmeth,'eigs')) % compute p.nc.neig FEM eigenvalues 
    try eigssol=p.sw.eigssol; catch; eigssol=0; end 
    switch eigssol
    case 0 % STANDARD 
     if p.sw.eigsstart==1; p.sw.evopts.v0=ones(vs,1)/vs;  
     else; try; p.sw.evopts=rmfield(p.sw.evopts,'v0'); catch; end; end 
     [V,mu]=eigs(Gu,M,neig,om,p.sw.evopts);    
    case 1 % global coupling version with global vars 
      opts.tol=1e-4; opts.p=2*p.nc.neig+1; n=size(M,1); 
      [V,mu]=eigs(@(b) gcafun(p,Gu,M,om,b),n,M,neig,om,opts); 
    end
    muv=mu*ones(1,neig)'; 
else % compute Evals with real-part in [p.nc.eigint(1), p.nc.eigint(2)], usually slow 
    evalc('[V,muv]=sptarn(Gu,M,p.nc.eigint(1),p.nc.eigint(2));');
end
mul=length(muv); [ms,ix]=sort(abs(real(muv))); Vs=zeros(vs,mul);
ineg=0; mus=zeros(1,mul); 
try intol=p.nc.intol; catch intol=0; end 
for i=1:mul; mus(i)=muv(ix(i)); Vs(:,i)=V(:,ix(i));
    if (mus(i)<intol) ineg=ineg+1; end
end; 
if p.sw.verb>1; figure(5+j); clf; plot(real(mus), imag(mus),'*'); end 
minr=min(real(muv)); maxr=max(real(muv)); 
if (p.sw.verb>2 && abs(minr)>abs(maxr)/2)
    fprintf('warning: mu-min=%g, mu-max=%g; consider increasing p.nc.neig\n', mus(1),mus(mul));
end
muv=mus; V=Vs; 

if (p.sw.spcont~=0); p=spextend(p,spcont,prim); end % reset to spectral cont
end

